import 'package:example/main.dart';
import 'package:example/storage/session_storage.dart'
    if (dart.library.html) 'package:example/storage/session_storage_html.dart';
import 'package:flutter/material.dart';
import 'package:flutterx_live_data/flutterx_live_data.dart';

class MutableLiveDataTest extends StatefulWidget {
  const MutableLiveDataTest({super.key});

  @override
  State<MutableLiveDataTest> createState() => _MutableLiveDataTestState();
}

class _MutableLiveDataTestState extends State<MutableLiveDataTest> {
  final MutableLiveData<int> _x = MutableLiveData(value: 0);
  final MutableLiveData<int> _xl = MutableLiveData.late();
  late final MediatorLiveData<int> _y = MediatorLiveData()..addTransformSource<int>(_x, (x) => x * 10);
  late final MediatorMutableLiveData<int> _ym = MediatorMutableLiveData(value: 0)
    ..addTransformSource<int>(_x, (x) => x * 10);
  final HistoryMutableLiveData<int> _h = HistoryMutableLiveData(value: 0);
  final StorageMutableLiveData<int> _s = StorageMutableLiveData.int(key: 's', defaultValue: 0, storage: sessionStorage);

  @override
  Widget build(BuildContext context) => Row(children: [
        xColumn('Data', const [
          Text('X [mutable]'),
          Text('Xl [mutable,late]'),
          Text('Y [mediator =x*10]'),
          Text('Ym [mutable,mediator =x*10]'),
          Text('h [mutable,history]'),
          Text('s [mutable,storage]'),
        ]),
        const SizedBox(width: kMinInteractiveDimension),
        xColumn('Value', [
          xData(_x),
          xData(_xl),
          xData(_y),
          xData(_ym),
          xData(_h),
          xData(_s),
        ]),
        const SizedBox(width: kMinInteractiveDimension),
        xColumn('Actions', [
          xAction(onPressed: () => _x.value++, icon: Icons.add),
          xAction(onPressed: () => _xl.initialized ? _xl.value++ : _xl.value = 0, icon: Icons.add),
          const SizedBox(height: 20),
          xAction(onPressed: () => _ym.initialized ? _ym.value++ : _ym.value = 0, icon: Icons.add),
          xActions([
            xAction(onPressed: () => _h.value++, icon: Icons.add),
            LiveDataBuilder<bool>(
                data: _h.canUndo,
                builder: (context, enabled) => xAction(onPressed: enabled ? _h.historyUndo : null, icon: Icons.undo)),
            LiveDataBuilder<bool>(
                data: _h.canRedo,
                builder: (context, enabled) => xAction(onPressed: enabled ? _h.historyRedo : null, icon: Icons.redo)),
          ]),
          xActions([
            xAction(onPressed: () => _s.value++, icon: Icons.add),
            xAction(onPressed: _s.reset, icon: Icons.delete),
          ]),
        ]),
      ]);
}
