part of '../flutterx_live_data.dart';

extension ObservableDataExt<T> on ObservableData<T> {
  T? get valueOrNull => initialized ? _data : null;

  MediatorLiveData<Y> transform<Y>(FutureOr<Y> Function(T) transform, {bool reactive = true}) => MediatorLiveData<Y>()
    ..reactive = reactive
    ..addTransformSource<T>(this, transform);

  void observe(ObserverMixin context, Observer<T> onChanged, {Dispatcher? dispatcher}) =>
      context._observe<T>(this, onChanged, dispatcher);

  Future<T> whenValue({
    bool Function(T)? accept,
    Dispatcher? dispatcher,
    Duration? timeLimit,
    FutureOr<T> Function()? onTimeout,
    Future<T>? cancel,
  }) {
    final completer = Completer<T>();
    final observer = accept == null
        ? completer.complete as Observer<T>
        : (value) {
            if (accept(value)) completer.complete(value);
          };
    observeForever(observer, dispatcher: dispatcher);
    cancel?.then(completer.complete, onError: completer.completeError);
    return (timeLimit == null ? completer.future : completer.future.timeout(timeLimit, onTimeout: onTimeout))
        .whenComplete(() => removeObserver(observer));
  }
}

extension MediatorExt<Y> on MediatorMixin<Y> {
  void addTransformSource2<X1, X2>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    FutureOr<Y> Function(X1, X2) transform,
  ) =>
      addTransformSources([x1, x2], (values) => transform(values[0], values[1]));

  void addTransformSource3<X1, X2, X3>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    ObservableData<X3> x3,
    FutureOr<Y> Function(X1, X2, X3) transform,
  ) =>
      addTransformSources([x1, x2, x3], (values) => transform(values[0], values[1], values[2]));

  void addTransformSource4<X1, X2, X3, X4>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    ObservableData<X3> x3,
    ObservableData<X4> x4,
    FutureOr<Y> Function(X1, X2, X3, X4) transform,
  ) =>
      addTransformSources([x1, x2, x3, x4], (values) => transform(values[0], values[1], values[2], values[3]));

  void addTransformSource5<X1, X2, X3, X4, X5>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    ObservableData<X3> x3,
    ObservableData<X4> x4,
    ObservableData<X5> x5,
    FutureOr<Y> Function(X1, X2, X3, X4, X5) transform,
  ) =>
      addTransformSources(
          [x1, x2, x3, x4, x5], (values) => transform(values[0], values[1], values[2], values[3], values[4]));

  Disposable addLiveTransformSource2<X1, X2>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    ObservableData<Y> Function(X1, X2) transform,
  ) =>
      addLiveTransformSources([x1, x2], (values) => transform(values[0], values[1]));

  Disposable addLiveTransformSource3<X1, X2, X3>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    ObservableData<X3> x3,
    ObservableData<Y> Function(X1, X2, X3) transform,
  ) =>
      addLiveTransformSources([x1, x2, x3], (values) => transform(values[0], values[1], values[2]));

  Disposable addLiveTransformSource4<X1, X2, X3, X4>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    ObservableData<X3> x3,
    ObservableData<X4> x4,
    ObservableData<Y> Function(X1, X2, X3, X4) transform,
  ) =>
      addLiveTransformSources([x1, x2, x3, x4], (values) => transform(values[0], values[1], values[2], values[3]));

  Disposable addLiveTransformSource5<X1, X2, X3, X4, X5>(
          ObservableData<X1> x1,
          ObservableData<X2> x2,
          ObservableData<X3> x3,
          ObservableData<X4> x4,
          ObservableData<X5> x5,
          ObservableData<Y> Function(X1, X2, X3, X4, X5) transform) =>
      addLiveTransformSources(
          [x1, x2, x3, x4, x5], (values) => transform(values[0], values[1], values[2], values[3], values[4]));
}

extension ObserverMixinExt on ObserverMixin {
  void observe<X>(ObservableData<X> x, Observer<X> onChanged, {Dispatcher? dispatcher}) =>
      _observe<X>(x, onChanged, dispatcher);

  void observe2<X1, X2>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    void Function(X1, X2) onChanged, {
    Dispatcher? dispatcher,
  }) =>
      _observeAll([x1, x2], onChanged, (values) => onChanged(values[0], values[1]), dispatcher);

  void observe3<X1, X2, X3>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    ObservableData<X3> x3,
    void Function(X1, X2, X3) onChanged, {
    Dispatcher? dispatcher,
  }) =>
      _observeAll([x1, x2, x3], onChanged, (values) => onChanged(values[0], values[1], values[2]), dispatcher);

  void observe4<X1, X2, X3, X4>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    ObservableData<X3> x3,
    ObservableData<X4> x4,
    void Function(X1, X2, X3, X4) onChanged, {
    Dispatcher? dispatcher,
  }) =>
      _observeAll(
          [x1, x2, x3, x4], onChanged, (values) => onChanged(values[0], values[1], values[2], values[3]), dispatcher);

  void observe5<X1, X2, X3, X4, X5>(
    ObservableData<X1> x1,
    ObservableData<X2> x2,
    ObservableData<X3> x3,
    ObservableData<X4> x4,
    ObservableData<X5> x5,
    void Function(X1, X2, X3, X4, X5) onChanged, {
    Dispatcher? dispatcher,
  }) =>
      _observeAll([x1, x2, x3, x4, x5], onChanged,
          (values) => onChanged(values[0], values[1], values[2], values[3], values[4]), dispatcher);

  void observeAll(
    Iterable<ObservableData> sources,
    void Function(List values) onChanged, {
    Dispatcher? dispatcher,
  }) =>
      _observeAll([...sources], onChanged, onChanged, dispatcher);
}

Disposable observe2Forever<X1, X2>(
  ObservableData<X1> x1,
  ObservableData<X2> x2,
  void Function(X1, X2) onChanged, {
  Dispatcher? dispatcher,
}) =>
    observeAllForever([x1, x2], (values) => onChanged(values[0], values[1]), dispatcher: dispatcher);

Disposable observe3Forever<X1, X2, X3>(
  ObservableData<X1> x1,
  ObservableData<X2> x2,
  ObservableData<X3> x3,
  void Function(X1, X2, X3) onChanged, {
  Dispatcher? dispatcher,
}) =>
    observeAllForever([x1, x2, x3], (values) => onChanged(values[0], values[1], values[2]), dispatcher: dispatcher);

Disposable observe4Forever<X1, X2, X3, X4>(
  ObservableData<X1> x1,
  ObservableData<X2> x2,
  ObservableData<X3> x3,
  ObservableData<X4> x4,
  void Function(X1, X2, X3, X4) onChanged, {
  Dispatcher? dispatcher,
}) =>
    observeAllForever([x1, x2, x3, x4], (values) => onChanged(values[0], values[1], values[2], values[3]),
        dispatcher: dispatcher);

Disposable observe5Forever<X1, X2, X3, X4, X5>(
  ObservableData<X1> x1,
  ObservableData<X2> x2,
  ObservableData<X3> x3,
  ObservableData<X4> x4,
  ObservableData<X5> x5,
  void Function(X1, X2, X3, X4, X5) onChanged, {
  Dispatcher? dispatcher,
}) =>
    observeAllForever(
        [x1, x2, x3, x4, x5], (values) => onChanged(values[0], values[1], values[2], values[3], values[4]),
        dispatcher: dispatcher);

Disposable observeAllForever(
  Iterable<ObservableData> sources,
  void Function(List values) onChanged, {
  Dispatcher? dispatcher,
}) {
  final observers = <ObserverWrapper>[];
  ObservableData.observeAll(sources, onChanged,
      <V>(source, onChanged) => observers.add(source.observeForever(onChanged, dispatcher: dispatcher)));
  return _Disposable(() {
    for (final observer in observers) observer.dispose();
  });
}
