part of '../flutterx_live_data.dart';

typedef Observer<T> = void Function(T value);

abstract class Disposable {
  const Disposable._();

  void dispose();
}

class ObserverWrapper<T> implements Disposable {
  ObservableData<T>? _data;
  @protected
  final Observer<T> onChanged;
  @protected
  final Dispatcher? dispatcher;

  bool get active => _data != null;

  ObserverWrapper(this.onChanged, this.dispatcher);

  @protected
  void onAdded(ObservableData<T> data) {}

  @protected
  void onRemoved(ObservableData<T> data) {}

  @override
  void dispose() => _data?.removeObserver(onChanged);

  @override
  String toString() => '${objectRuntimeType(this, 'ObserverWrapper')}#$hashCode';

  void _attach(ObservableData<T> data) {
    assert(_data == null, 'observer already attached');
    _lastVersion = -1;
    _data = data;
    onAdded(data);
    data._onObserverAttach(this);
  }

  void _detach(ObservableData<T> data) {
    assert(identical(_data, data), 'observer not attached to $data');
    data._onObserverDetach(this);
    onRemoved(data);
    _data = null;
  }

  int _lastVersion = -1;
  late T _pendingData;
  bool _dispatchingPendingData = false;

  void _considerNotify() {
    final data = _data;
    if (data != null && data._version > _lastVersion) {
      _lastVersion = data._version;
      final dispatcher = this.dispatcher;
      if (dispatcher == null) return onChanged(data._data);
      _pendingData = data._data;
      if (_dispatchingPendingData) return;
      _dispatchingPendingData = true;
      dispatcher.post(() {
        _dispatchingPendingData = false;
        if (identical(_data, data)) onChanged(_pendingData);
      });
    }
  }
}

@immutable
class _Disposable implements Disposable {
  final VoidCallback _dispose;

  const _Disposable(this._dispose);

  @override
  void dispose() => _dispose();
}
