part of '../flutterx_live_data.dart';

class LiveResultBuilder<T> extends StatelessWidget {
  final LiveResult<T> result;
  final WidgetBuilder idleBuilder;
  final WidgetBuilder loaderBuilder;
  final LiveWidgetBuilder<T> successBuilder;
  final LiveWidgetBuilder errorBuilder;

  const LiveResultBuilder({
    super.key,
    required this.result,
    required this.idleBuilder,
    required this.loaderBuilder,
    required this.successBuilder,
    required this.errorBuilder,
  });

  @override
  StatelessElement createElement() => _LiveResultBuilderElement<T>(this);

  @override
  Widget build(BuildContext context) {
    switch (result.state) {
      case ResultState.idle:
        return idleBuilder(context);
      case ResultState.loading:
        return loaderBuilder(context);
      case ResultState.error:
        return errorBuilder(context, result._value);
      case ResultState.success:
        return successBuilder(context, result._value);
    }
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) => super.debugFillProperties(properties
    ..add(DiagnosticsProperty<LiveResult<T>>('result', result))
    ..add(ObjectFlagProperty<WidgetBuilder>.has('idleBuilder', idleBuilder))
    ..add(ObjectFlagProperty<WidgetBuilder>.has('loaderBuilder', loaderBuilder))
    ..add(ObjectFlagProperty<LiveWidgetBuilder<T>>.has('successBuilder', successBuilder))
    ..add(ObjectFlagProperty<LiveWidgetBuilder>.has('errorBuilder', errorBuilder)));
}

class _LiveResultBuilderElement<T> extends StatelessElement {
  @override
  LiveResultBuilder<T> get widget => super.widget as LiveResultBuilder<T>;

  _LiveResultBuilderElement(LiveResultBuilder<T> super.widget);

  @override
  void mount(Element? parent, Object? newSlot) {
    widget.result.observeForever(_onChanged, dispatcher: Dispatcher.postFrame);
    super.mount(parent, newSlot);
  }

  @override
  void update(LiveResultBuilder<T> newWidget) {
    final oldWidget = widget;
    if (!identical(oldWidget.result, newWidget.result)) {
      oldWidget.result.removeObserver(_onChanged);
      newWidget.result.observeForever(_onChanged, dispatcher: Dispatcher.postFrame);
    }
    super.update(newWidget);
  }

  @override
  void unmount() {
    widget.result.removeObserver(_onChanged);
    super.unmount();
  }

  void _onChanged(_) => markNeedsBuild();
}
